<?php

namespace Drupal\mcapi_forms\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\migrate\Event\MigratePreRowSaveEvent;
use Drupal\migrate\MigrateSkipRowException;
use Drupal\migrate\Event\MigratePostRowSaveEvent;

/**
 * Subscriber to create a router item for each transaction form display.
 */
class MigrationSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() : array {
    return [
      'migrate.pre_row_save' => [['migratePreRowSave']],
      'migrate.post_row_save' => [['migratePostRowSave']]
    ];
  }

  /**
   *
   * @param MigratePreRowSaveEvent $event
   */
  public function migratePreRowSave(MigratePreRowSaveEvent $event) {
    $row = $event->getRow();
    switch ($event->getMigration()->id()) {
      case 'd7_filter_format':
        if ($row->getSourceProperty('format') == 'mcapiform_string_format') {
          return new MigrateSkipRowException('Deprecated filter format');
        }
        break;

      case 'd7_menu_links':
        $link_path = $row->getSourceProperty('link_path');
        if ($link_path == 'transact/3rdparty')   {
          $row->setDestinationProperty('link_path', 'transaction/add');
        }
        if ($link_path == 'transact/1stparty')   {
          $row->setDestinationProperty('link_path', 'transact/credit');
        }
    }
  }

  function migratePostRowSave(MigratePostRowSaveEvent $event) {
    if ($event->getMigration()->id() == 'd7_mcapi_form') {
      $event->logMessage('Route access on all custom transaction forms must be re-configured.', 'warning');
    }
  }

}
